@@ -18,15 +18,20 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
+import functools
+import types
+
 from twisted.internet.defer import Deferred, succeed
 
 from qt import QPropertyAnimation, QParallelAnimationGroup, \
     QAbstractAnimation, QEasingCurve, QVariant
 
-from common import Internal, Debug, isAlive
+from common import Internal, Debug, isAlive, isPython3, nativeString
 from log import logDebug
 
+
 class Animation(QPropertyAnimation):
+
     """a Qt4 animation with helper methods"""
 
     nextAnimations = []
@@ -42,35 +47,42 @@ class Animation(QPropertyAnimation):
         if uiTile.tile in Debug.animation:
             oldAnimation = uiTile.activeAnimation.get(propName, None)
             if isAlive(oldAnimation):
-                logDebug('new animation %s (after %s is done)' % (self, oldAnimation.ident()))
+                logDebug(
+                    u'new animation %s (after %s is done)' %
+                    (self, oldAnimation.ident()))
             else:
-                logDebug('new animation %s' % self)
+                logDebug(u'new animation %s' % self)
 
     def setEndValue(self, endValue):
         """wrapper with debugging code"""
         uiTile = self.targetObject()
         if uiTile.tile in Debug.animation:
             pName = self.pName()
-            logDebug('%s: change endValue for %s: %s->%s  %s' % (self.ident(), pName, self.formatValue(self.endValue()),
-                    self.formatValue(endValue), uiTile))
+            logDebug(
+                u'%s: change endValue for %s: %s->%s  %s' % (self.ident(), pName, self.formatValue(self.endValue()),
+                                                             self.formatValue(endValue), uiTile))
         QPropertyAnimation.setEndValue(self, endValue)
 
     def ident(self):
         """the identifier to be used in debug messages"""
         pGroup = self.group()
         if pGroup:
-            return '%d/A%d' % (id(pGroup)%10000, id(self) % 10000)
+            return '%d/A%d' % (id(pGroup) % 10000, id(self) % 10000)
         else:
             return 'A%d' % (id(self) % 10000)
 
     def pName(self):
-        """return self.propertyName() as a python string"""
-        return str(self.propertyName())
+        """
+        Return self.propertyName() as a python string.
+
+        @return: C{str}
+        """
+        return nativeString(self.propertyName())
 
     def unpackValue(self, qvariant):
         """get the wanted value from the QVariant"""
         if not isinstance(qvariant, QVariant):
-            return qvariant # is already autoconverted
+            return qvariant  # is already autoconverted
         pName = self.pName()
         if pName == 'pos':
             return qvariant.toPointF()
@@ -98,12 +110,14 @@ class Animation(QPropertyAnimation):
     def __str__(self):
         """for debug messages"""
         return '%s: %s->%s for %s' % (self.ident(), self.pName(),
-            self.formatValue(self.endValue()), self.targetObject())
+                                      self.formatValue(self.endValue()), self.targetObject())
+
 
 class ParallelAnimationGroup(QParallelAnimationGroup):
+
     """override __init__"""
 
-    running = [] # we need a reference to active animation groups
+    running = []  # we need a reference to active animation groups
     current = None
 
     def __init__(self, parent=None):
@@ -116,8 +130,8 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
         self.debug = False
         if ParallelAnimationGroup.current:
             if self.debug or ParallelAnimationGroup.current.debug:
-                logDebug('Chaining Animation group %d to %d' % \
-                        (id(self), id(ParallelAnimationGroup.current)))
+                logDebug(u'Chaining Animation group %d to %d' %
+                         (id(self), id(ParallelAnimationGroup.current)))
             ParallelAnimationGroup.current.deferred.addCallback(self.start)
         else:
             self.start()
@@ -160,10 +174,12 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
         self.finished.connect(self.allFinished)
         scene = Internal.scene
         scene.focusRect.hide()
-        QParallelAnimationGroup.start(self, QAbstractAnimation.DeleteWhenStopped)
+        QParallelAnimationGroup.start(
+            self,
+            QAbstractAnimation.DeleteWhenStopped)
         if self.debug:
-            logDebug('Animation group %d started (%s)' % (
-                    id(self), ','.join('A%d' % (id(x) % 10000) for x in self.animations)))
+            logDebug(u'Animation group %d started (%s)' % (
+                id(self), ','.join('A%d' % (id(x) % 10000) for x in self.animations)))
         return succeed(None)
 
     def allFinished(self):
@@ -175,12 +191,12 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
         if Debug.animationSpeed and self.duration():
             perSecond = self.steps * 1000.0 / self.duration()
             if perSecond < 50:
-                logDebug('%d steps for %d animations, %.1f/sec' % \
-                (self.steps, len(self.children()), perSecond))
+                logDebug(u'%d steps for %d animations, %.1f/sec' %
+                         (self.steps, len(self.children()), perSecond))
         # if we have a deferred, callback now
         assert self.deferred
         if self.debug:
-            logDebug('Animation group %d done' % id(self))
+            logDebug(u'Animation group %d done' % id(self))
         if self.deferred:
             self.deferred.callback(None)
 
@@ -194,9 +210,12 @@ class ParallelAnimationGroup(QParallelAnimationGroup):
             Internal.scene.focusRect.refresh()
         return
 
-class Animated(object):
+
+class MoveImmediate(object):
+
     """a helper class for moving tiles with or without animation"""
-    def __init__(self, animateMe=True):
+
+    def __init__(self, animateMe=False):
         if Internal.Preferences:
             self.__animateMe = animateMe
             self.prevAnimationSpeed = Internal.Preferences.animationSpeed
@@ -214,18 +233,37 @@ class Animated(object):
                 Internal.Preferences.animationSpeed = self.prevAnimationSpeed
 
 
-def afterCurrentAnimationDo(callback, *args, **kwargs):
+def __afterCurrentAnimationDo(callback, *args, **kwargs):
     """a helper, delaying some action until all active
     animations have finished"""
     current = ParallelAnimationGroup.current
     if current:
         current.deferred.addCallback(callback, *args, **kwargs)
         if current.debug:
-            logDebug('after current animation %d do %s %s' % \
-                (id(current), callback, ','.join(args) if args else ''))
+            logDebug(u'after current animation %d do %s %s' %
+                     (id(current), callback, ','.join(args) if args else ''))
     else:
         callback(None, *args, **kwargs)
 
+
+def afterQueuedAnimations(f):
+    """A decorator"""
+
+    @functools.wraps(f)
+    def doAfterQueuedAnimations(*args, **kwargs):
+        animate()
+        method = types.MethodType(f, args[0])
+        args = args[1:]
+        if isPython3:
+            assert f.__code__.co_varnames[
+                1] == 'deferredResult', f.__qualname__
+        else:
+            assert f.func_code.co_varnames[1] == 'deferredResult', f.__name__
+        return __afterCurrentAnimationDo(method, *args, **kwargs)
+
+    return doAfterQueuedAnimations
+
+
 def animate():
     """now run all prepared animations. Returns a Deferred
         so callers can attach callbacks to be executed when
@@ -237,10 +275,11 @@ def animate():
     """
     if Animation.nextAnimations:
         shortcutMe = (Internal.scene is None
-                or Internal.mainWindow.centralView.dragObject
-                or Internal.Preferences.animationSpeed == 99
-                or len(Animation.nextAnimations) > 1000)
-                # change 1000 to 100 if we do not want to animate shuffling and initial deal
+                      or Internal.mainWindow.centralView.dragObject
+                      or Internal.Preferences.animationSpeed == 99
+                      or len(Animation.nextAnimations) > 1000)
+                # change 1000 to 100 if we do not want to animate shuffling and
+                # initial deal
         if not shortcutMe:
             duration = 0
             for animation in Animation.nextAnimations:
