@@ -18,39 +18,42 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-from qt import Qt, QVariant, QAbstractTableModel, QModelIndex, QSize
+from qt import Qt, toQVariant, QAbstractTableModel, QModelIndex, QSize
 from qt import QWidget, QLineEdit, QVBoxLayout, QColor, QAbstractItemView
 
 from log import m18n, logDebug
 from guiutil import MJTableView, decorateWindow
 from statesaver import StateSaver
 from message import ChatMessage
-from common import Debug
+from common import Debug, unicode
 from modeltest import ModelTest
 from kde import KApplication
 
+
 class ChatModel(QAbstractTableModel):
+
     """a model for the chat view"""
+
     def __init__(self, parent=None):
         super(ChatModel, self).__init__(parent)
         self.chatLines = []
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole): # pylint: disable=no-self-use
+    def headerData(self, section, orientation, role=Qt.DisplayRole):  # pylint: disable=no-self-use
         """show header"""
         if role == Qt.TextAlignmentRole:
             if orientation == Qt.Horizontal:
                 if section == 1:
-                    return QVariant(int(Qt.AlignRight))
+                    return toQVariant(int(Qt.AlignRight))
                 else:
-                    return QVariant(int(Qt.AlignLeft))
+                    return toQVariant(int(Qt.AlignLeft))
         if orientation != Qt.Horizontal:
-            return QVariant(int(section+1))
+            return toQVariant(int(section + 1))
         if role != Qt.DisplayRole:
-            return QVariant()
+            return toQVariant()
         result = ''
         if section < self.columnCount():
             result = [m18n('Time'), m18n('Player'), m18n('Message')][section]
-        return QVariant(result)
+        return toQVariant(result)
 
     def rowCount(self, parent=None):
         """how many lines are in the model?"""
@@ -59,64 +62,79 @@ class ChatModel(QAbstractTableModel):
             return 0
         return len(self.chatLines)
 
-    def columnCount(self, dummyParent=None): # pylint: disable=no-self-use
+    def columnCount(self, dummyParent=None):  # pylint: disable=no-self-use
         """for now we only have time, who, message"""
         return 3
 
     def data(self, index, role=Qt.DisplayRole):
         """score table"""
-        result = QVariant()
+        result = toQVariant()
         if role == Qt.TextAlignmentRole:
             if index.column() == 1:
-                return QVariant(int(Qt.AlignRight))
+                return toQVariant(int(Qt.AlignRight))
             else:
-                return QVariant(int(Qt.AlignLeft))
+                return toQVariant(int(Qt.AlignLeft))
         if index.isValid() and (0 <= index.row() < len(self.chatLines)):
             chatLine = self.chatLines[index.row()]
             if role == Qt.DisplayRole and index.column() == 0:
                 local = chatLine.localtimestamp()
-                result = QVariant('%02d:%02d:%02d' % (
+                result = toQVariant('%02d:%02d:%02d' % (
                     local.hour,
                     local.minute,
                     local.second))
             elif role == Qt.DisplayRole and index.column() == 1:
-                result = QVariant(chatLine.fromUser)
+                result = toQVariant(chatLine.fromUser)
             elif role == Qt.DisplayRole and index.column() == 2:
-                result = QVariant(m18n(chatLine.message))
+                result = toQVariant(m18n(chatLine.message))
             elif role == Qt.ForegroundRole and index.column() == 2:
                 palette = KApplication.palette()
-                color = 'blue' if chatLine.isStatusMessage else palette.windowText()
-                result = QVariant(QColor(color))
+                color = 'blue' if chatLine.isStatusMessage else palette.windowText(
+                )
+                result = toQVariant(QColor(color))
         return result
 
     def appendLines(self, lines):
         """insert a chatline"""
         if not isinstance(lines, list):
             lines = [lines]
-        self.beginInsertRows(QModelIndex(), self.rowCount(), self.rowCount() + len(lines) - 1)
+        self.beginInsertRows(
+            QModelIndex(),
+            self.rowCount(),
+            self.rowCount() + len(lines) - 1)
         self.chatLines.extend(lines)
         self.endInsertRows()
 
+
 class ChatView(MJTableView):
+
     """define a minimum size"""
+
     def __init__(self):
         MJTableView.__init__(self)
-    def sizeHint(self): # pylint: disable=no-self-use
+
+    def sizeHint(self):  # pylint: disable=no-self-use
         """sizeHint"""
         return QSize(400, 100)
+
     def minimumSizeHint(self):
         """minimumSizeHint"""
         return self.sizeHint()
 
+
 class ChatWindow(QWidget):
+
     """a widget for showing chat messages"""
+
     def __init__(self, scene=None, table=None):
         super(ChatWindow, self).__init__(None)
         self.scene = scene
         self.table = table or scene.game.client.table
         self.table.chatWindow = self
         self.setObjectName('chatWindow')
-        title = m18n('Chat on table %1 at %2', self.table.tableid, self.table.client.connection.url)
+        title = m18n(
+            'Chat on table %1 at %2',
+            self.table.tableid,
+            self.table.client.connection.url)
         decorateWindow(self, title)
         self.messageView = ChatView()
         self.messageView.setModel(ChatModel())
@@ -125,7 +143,9 @@ class ChatWindow(QWidget):
         self.messageView.setWordWrap(False)
         self.messageView.setSelectionMode(QAbstractItemView.NoSelection)
         if Debug.modelTest:
-            self.debugModelTest = ModelTest(self.messageView.model(), self.messageView)
+            self.debugModelTest = ModelTest(
+                self.messageView.model(),
+                self.messageView)
         self.edit = QLineEdit()
         layout = QVBoxLayout()
         layout.addWidget(self.messageView)
@@ -150,7 +170,7 @@ class ChatWindow(QWidget):
     def kill(self):
         """hide and null on table"""
         if Debug.chat:
-            logDebug('chat.kill for %s on table %s' % (self, self.table))
+            logDebug(u'chat.kill for %s on table %s' % (self, self.table))
         self.hide()
         self.table.chatWindow = None
 
@@ -161,8 +181,12 @@ class ChatWindow(QWidget):
             self.edit.clear()
         if line:
             if Debug.chat:
-                logDebug('sending line %s to others' % line)
-            msg = ChatMessage(self.table.tableid, self.table.client.name, line, isStatusMessage)
+                logDebug(u'sending line %s to others' % line)
+            msg = ChatMessage(
+                self.table.tableid,
+                self.table.client.name,
+                line,
+                isStatusMessage)
             self.table.client.sendChat(msg).addErrback(self.chatError)
 
     def chatError(self, result):
@@ -180,6 +204,8 @@ class ChatWindow(QWidget):
         self.show()
         self.messageView.model().appendLines(chatLine)
         for row in range(self.messageView.model().rowCount()):
-            self.messageView.setRowHeight(row, self.messageView.fontMetrics().height())
+            self.messageView.setRowHeight(
+                row,
+                self.messageView.fontMetrics().height())
         self.messageView.resizeColumnsToContents()
         self.messageView.scrollToBottom()
