@@ -20,16 +20,23 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 # pylint: disable=unused-import
 
-import os, sys, shutil
+import os
+import sys
+import shutil
+import sip
 
-from common import Internal, Options
+from common import Internal, Options, unicode, isPython3
 
-usingKDE = False # pylint: disable=invalid-name
+usingKDE = False  # pylint: disable=invalid-name
 
 try:
-    if '--nokde' in sys.argv or '--qt5' in sys.argv:
+    if '--nokde' in sys.argv or '--qt5' in sys.argv or sip.SIP_VERSION == 0x041201:
         # for now there is no PyKDE5
         raise ImportError
+    if isPython3:
+        # PyKDE4 for python3 is unusable, fall back to qt only
+        # see KDE bug 253123
+        raise ImportError
     from PyKDE4.kdecore import KUser, KGlobal, KStandardDirs, \
         KAboutData, KCmdLineArgs, KConfig, KCmdLineOptions
     from PyKDE4.kdecore import i18n, i18nc, ki18n
@@ -37,7 +44,9 @@ try:
         KConfigSkeleton, KDialogButtonBox, KAction, KStandardAction, \
         KApplication, KToggleFullScreenAction, KXmlGuiWindow, \
         KConfigDialog, KDialog
-    usingKDE = True # pylint: disable=invalid-name
+    usingKDE = True  # pylint: disable=invalid-name
+    KDialog.NoButton = getattr(KDialog, 'None')
+    # see https://bugs.kde.org/show_bug.cgi?id=333683
 except ImportError:
     try:
         raise
@@ -46,11 +55,14 @@ except ImportError:
     except ImportError:
         from kdestub import *  # pylint: disable=wildcard-import
 
-if usingKDE:
-    KDialog.NoButton = KDialog.None
 
 def appdataDir():
-    """the per user directory with kajongg application information like the database"""
+    """
+    The per user directory with kajongg application information like the database.
+
+    @return: The directory path.
+    @rtype: C{str}.
+    """
     serverDir = os.path.expanduser('~/.kajonggserver/')
     if Internal.isServer:
         # the server might or might not have KDE installed, so to be on
@@ -58,9 +70,12 @@ def appdataDir():
         # the following code moves an existing kajonggserver.db to .kajonggserver
         # but only if .kajonggserver does not yet exist
         kdehome = os.environ.get('KDEHOME', '~/.kde')
-        oldPath = os.path.expanduser(kdehome + '/share/apps/kajongg/kajonggserver.db')
+        oldPath = os.path.expanduser(
+            kdehome +
+            '/share/apps/kajongg/kajonggserver.db')
         if not os.path.exists(oldPath):
-            oldPath = os.path.expanduser('~/.kde4/share/apps/kajongg/kajonggserver.db')
+            oldPath = os.path.expanduser(
+                '~/.kde4/share/apps/kajongg/kajonggserver.db')
         if os.path.exists(oldPath) and not os.path.exists(serverDir):
             # upgrading an old kajonggserver installation
             os.makedirs(serverDir)
@@ -75,15 +90,18 @@ def appdataDir():
         if not os.path.exists(serverDir):
             # the client wants to place the socket in serverDir
             os.makedirs(serverDir)
-        result = os.path.dirname(unicode(KGlobal.dirs().locateLocal("appdata", ""))) + '/'
+        result = os.path.dirname(
+            unicode(KGlobal.dirs().locateLocal("appdata", ""))) + '/'
         return result
 
+
 def cacheDir():
     """the cache directory for this user"""
     if Internal.isServer:
         result = os.path.join(appdataDir(), 'cache')
     else:
-        result = os.path.dirname(unicode(KGlobal.dirs().locateLocal("cache", "")))
+        result = os.path.dirname(
+            unicode(KGlobal.dirs().locateLocal("cache", "")))
         result = os.path.join(result, 'kajongg')
     if not os.path.exists(result):
         try:
@@ -92,11 +110,14 @@ def cacheDir():
             pass
     return result
 
+
 def socketName():
     """client and server process use this socket to talk to each other"""
     serverDir = os.path.expanduser('~/.kajonggserver')
     if not os.path.exists(serverDir):
-        appdataDir() # allocate the directory and possibly move old databases there
+        appdataDir()
+                   # allocate the directory and possibly move old databases
+                   # there
     if Options.socket:
         return Options.socket
     else:
