@@ -20,18 +20,21 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 import weakref
 
-from common import Debug
+from common import Debug, unicodeString, StrMixin, nativeString
 from message import Message
 from tile import Tile, TileList
 from meld import Meld, MeldList
 
-class Move(object):
+
+class Move(StrMixin):
+
     """used for decoded move information from the game server"""
+
     def __init__(self, player, command, kwargs):
         if isinstance(command, Message):
             self.message = command
         else:
-            self.message = Message.defined[command]
+            self.message = Message.defined[nativeString(command)]
         self.table = None
         self.notifying = False
         self._player = weakref.ref(player) if player else None
@@ -45,13 +48,15 @@ class Move(object):
             if value is None:
                 self.__setattr__(key, None)
             elif key.lower().endswith('tile'):
-                self.__setattr__(key, Tile(value))
+                self.__setattr__(key, Tile(nativeString(value)))
             elif key.lower().endswith('tiles'):
-                self.__setattr__(key, TileList(value))
+                self.__setattr__(key, TileList(nativeString(value)))
             elif key.lower().endswith('meld'):
-                self.__setattr__(key, Meld(value))
+                self.__setattr__(key, Meld(nativeString(value)))
             elif key.lower().endswith('melds'):
-                self.__setattr__(key, MeldList(value))
+                self.__setattr__(key, MeldList(nativeString(value)))
+            elif key in ('wantedGame', 'score'):
+                self.__setattr__(key, nativeString(value))
             else:
                 self.__setattr__(key, value)
 
@@ -64,26 +69,30 @@ class Move(object):
     @staticmethod
     def prettyKwargs(kwargs):
         """this is also used by the server, but the server does not use class Move"""
-        result = ''
+        result = u''
         for key, value in kwargs.items():
             if key == 'token':
                 continue
+            if isinstance(value, (list, tuple)) and isinstance(value[0], (list, tuple)):
+                oldValue = value
+                tuples = []
+                for t in oldValue:
+                    tuples.append(u''.join(unicodeString(x) for x in t))
+                value = u','.join(tuples)
             if Debug.neutral and key == 'gameid':
-                result += ' gameid:GAMEID'
+                result += u' gameid:GAMEID'
             elif isinstance(value, bool) and value:
-                result += ' %s' % key
+                result += u' %s' % key
             elif isinstance(value, bool):
                 pass
-            elif isinstance(value, list) and isinstance(value[0], basestring):
-                result += ' %s:%s' % (key, ''.join(value))
+            elif isinstance(value, bytes):
+                result += u' %s:%s' % (key, unicodeString(value))
             else:
-                result += ' %s:%s' % (key, value)
-        result = result.replace("('", "(").replace("')", ")").replace(" '", "").replace(
-                "',", ",").replace("[(", "(").replace("])", ")")
+                result += u' %s:%s' % (key, value)
+        for old, new in ((u"('", u"("), (u"')", u")"), (u" '", u""),
+                         (u"',", u","), (u"[(", u"("), (u"])", u")")):
+            result = result.replace(old, new)
         return result
 
     def __unicode__(self):
         return u'%s %s%s' % (self.player, self.message, Move.prettyKwargs(self.kwargs))
-
-    def __repr__(self):
-        return '<Move: %s>' % unicode(self)
