@@ -18,13 +18,16 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-import os, tarfile, subprocess, datetime
+import os
+import tarfile
+import subprocess
+import datetime
 from io import BytesIO
 from hashlib import md5
 if os.name == 'nt':
-    import winsound # pylint: disable=import-error
+    import winsound  # pylint: disable=import-error
 
-from common import Debug, Internal
+from common import Debug, Internal, nativeString
 from util import which, removeIfExists, uniqueList, elapsedSince
 from log import logWarning, m18n, logDebug, logException
 
@@ -42,10 +45,11 @@ from tile import Tile
         # self.audio.enqueue(Phonon.MediaSource(wavName))
         # self.audio.play()
 
+
 class Sound(object):
+
     """the sound interface. Use class variables and class methods,
     thusly ensuring no two instances try to speak"""
-    enabled = False
     __oggName = None
     playProcesses = []
     lastCleaned = None
@@ -55,23 +59,30 @@ class Sound(object):
         """sets __oggName to exe name or False"""
         if Sound.__oggName is None:
             if os.name == 'nt':
-                parentDirectories = KGlobal.dirs().findDirs("appdata", "voices")
+                parentDirectories = KGlobal.dirs().findDirs(
+                    "appdata", "voices")
                 if parentDirectories:
                     oggName = os.path.join(parentDirectories[0], 'oggdec.exe')
-                    msg = '' # we bundle oggdec.exe with the kajongg installer, it must be there
+                    msg = ''  # we bundle oggdec.exe with the kajongg installer, it must be there
                 else:
-                    msg = m18n('No voices will be heard because the program %1 is missing', 'oggdec.exe')
+                    msg = m18n(
+                        'No voices will be heard because the program %1 is missing',
+                        'oggdec.exe')
             else:
                 oggName = 'ogg123'
-                msg = m18n('No voices will be heard because the program %1 is missing', oggName)
+                msg = m18n(
+                    'No voices will be heard because the program %1 is missing',
+                    oggName)
             if which(oggName):
                 Sound.__oggName = oggName
             else:
                 Sound.__oggName = False
-                Sound.enabled = False
+                Internal.Preferences.useSounds = False
                 # checks again at next reenable
                 if msg:
                     logWarning(msg)
+            if Debug.sound:
+                logDebug('ogg123 found:' + Sound.__oggName)
         return Sound.__oggName
 
     @staticmethod
@@ -106,7 +117,7 @@ class Sound(object):
     @staticmethod
     def speak(what):
         """this is what the user of this module will call."""
-        if not Sound.enabled:
+        if not Internal.Preferences.useSounds:
             return
         game = Internal.scene.game
         reactor = Internal.reactor
@@ -125,15 +136,19 @@ class Sound(object):
                     name, ext = os.path.splitext(what)
                     assert ext == '.ogg'
                     nameParts = os.path.normpath(name).split(os.sep)
-                    nameParts = nameParts[nameParts.index('voices')+1:]
-                    wavName = os.path.normpath('{}/{}.wav'.format(cacheDir(), '_'.join(nameParts)))
+                    nameParts = nameParts[nameParts.index('voices') + 1:]
+                    wavName = os.path.normpath(
+                        '{}/{}.wav'.format(cacheDir(),
+                                           '_'.join(nameParts)))
                     if not os.path.exists(wavName):
                         args = [oggName, '-a', '-w', wavName, what]
                         startupinfo = subprocess.STARTUPINFO()
                         startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                         subprocess.call(args, startupinfo=startupinfo)
                     try:
-                        winsound.PlaySound(wavName, winsound.SND_FILENAME | winsound.SND_NODEFAULT)
+                        winsound.PlaySound(
+                            wavName,
+                            winsound.SND_FILENAME | winsound.SND_NODEFAULT)
                     except RuntimeError:
                         pass
                 else:
@@ -162,10 +177,12 @@ class Sound(object):
             if len(text) == 2 and text[0] in 'sdbcw':
                 text = Tile(text).name()
             args = ['qdbus', 'org.kde.jovie',
-                '/KSpeech', 'say', text, '1']
+                    '/KSpeech', 'say', text, '1']
             subprocess.Popen(args)
 
+
 class Voice(object):
+
     """this administers voice sounds.
 
     When transporting voices between players, a compressed tarfile
@@ -180,7 +197,7 @@ class Voice(object):
         self.__md5sum = None
         if not os.path.split(directory)[0]:
             if Debug.sound:
-                logDebug('place voice %s in %s' % (directory, cacheDir()))
+                logDebug(u'place voice %s in %s' % (directory, cacheDir()))
             directory = os.path.join(cacheDir(), directory)
         self.directory = directory
         self.__setArchiveContent(content)
@@ -206,7 +223,7 @@ class Voice(object):
         else:
             logException('have neither HOME nor HOMEPATH')
         if home:
-            if self.directory.startswith(home.decode('utf-8')):
+            if self.directory.startswith(home):
                 return 'local'
         result = os.path.split(self.directory)[0]
         result = os.path.split(result)[1]
@@ -214,23 +231,25 @@ class Voice(object):
             result = 'en_US'
         return result
 
-
     @staticmethod
     def availableVoices():
         """a list of all voice directories"""
         if not Voice.__availableVoices:
             result = []
             for parentDirectory in KGlobal.dirs().findDirs("appdata", "voices"):
-                parentDirectory = unicode(parentDirectory)
+                parentDirectory = nativeString(parentDirectory)
                 for (dirpath, _, _) in os.walk(parentDirectory, followlinks=True):
                     if os.path.exists(os.path.join(dirpath, 's1.ogg')):
                         result.append(Voice(dirpath))
             group = KGlobal.config().group('Locale')
-            prefLanguages = uniqueList(':'.join(['local', str(group.readEntry('Language')), 'en_US']).split(':'))
-            prefLanguages = dict((x[1], x[0]) for x in enumerate(prefLanguages))
-            result = sorted(result, key=lambda x: prefLanguages.get(x.language(), 9999))
+            prefLanguages = uniqueList(
+                ':'.join(['local', str(group.readEntry('Language')), 'en_US']).split(':'))
+            prefLanguages = dict((x[1], x[0])
+                                 for x in enumerate(prefLanguages))
+            result = sorted(
+                result, key=lambda x: prefLanguages.get(x.language(), 9999))
             if Debug.sound:
-                logDebug('found voices:%s' % [str(x) for x in result])
+                logDebug(u'found voices:%s' % [str(x) for x in result])
             Voice.__availableVoices = result
         return Voice.__availableVoices
 
@@ -242,18 +261,26 @@ class Voice(object):
             dirname = os.path.split(voice.directory)[-1]
             if name == voice.md5sum:
                 if Debug.sound:
-                    logDebug('locate found %s by md5sum in %s' % (name, voice.directory))
+                    logDebug(
+                        u'locate found %s by md5sum in %s' %
+                        (name, voice.directory))
                 return voice
             elif name == dirname and voice.language() == 'local':
                 if Debug.sound:
-                    logDebug('locate found %s by name in %s' % (name, voice.directory))
+                    logDebug(
+                        u'locate found %s by name in %s' %
+                        (name, voice.directory))
                 return voice
         if Debug.sound:
-            logDebug('%s not found' % (name))
+            logDebug(u'%s not found' % (name))
 
     def buildSubvoice(self, oggName, side):
         """side is 'left' or 'right'."""
-        angleDirectory = os.path.join(cacheDir(), 'angleVoices', self.md5sum, side)
+        angleDirectory = os.path.join(
+            cacheDir(),
+            'angleVoices',
+            self.md5sum,
+            side)
         stdName = os.path.join(self.directory, oggName)
         angleName = os.path.join(angleDirectory, oggName)
         if os.path.exists(stdName) and not os.path.exists(angleName):
@@ -270,10 +297,12 @@ class Voice(object):
             callResult = subprocess.call(args)
             if callResult:
                 if Debug.sound:
-                    logDebug('failed to build subvoice %s: return code=%s' % (angleName, callResult))
+                    logDebug(
+                        u'failed to build subvoice %s: return code=%s' %
+                        (angleName, callResult))
                 return stdName
             if Debug.sound:
-                logDebug('built subvoice %s' % angleName)
+                logDebug(u'built subvoice %s' % angleName)
         return angleName
 
     def localTextName(self, text, angle):
@@ -292,7 +321,7 @@ class Voice(object):
         fileName = self.localTextName(text, angle)
         if not os.path.exists(fileName):
             if Debug.sound:
-                logDebug('Voice.speak: fileName %s not found' % fileName)
+                logDebug(u'Voice.speak: fileName %s not found' % fileName)
         Sound.speak(fileName)
 
     def oggFiles(self):
@@ -314,11 +343,12 @@ class Voice(object):
             removeIfExists(self.archiveName())
             removeIfExists(md5FileName)
             self.__md5sum = None
-            logDebug('no ogg files in %s' % self)
+            logDebug(u'no ogg files in %s' % self)
             return
         md5sum = md5()
         for oggFile in ogg:
-            md5sum.update(open(os.path.join(self.directory, oggFile)).read())
+            md5sum.update(
+                open(os.path.join(self.directory, oggFile), 'rb').read())
         # the md5 stamp goes into the old archive directory 'username'
         self.__md5sum = md5sum.hexdigest()
         existingMd5sum = self.savedmd5Sum()
@@ -326,15 +356,24 @@ class Voice(object):
         if self.__md5sum != existingMd5sum:
             if Debug.sound:
                 if not os.path.exists(md5Name):
-                    logDebug('creating new %s' % md5Name)
+                    logDebug(u'creating new %s' % md5Name)
                 else:
-                    logDebug('md5sum %s changed, rewriting %s with %s' % (existingMd5sum, md5Name, self.__md5sum))
+                    logDebug(
+                        u'md5sum %s changed, rewriting %s with %s' %
+                        (existingMd5sum, md5Name, self.__md5sum))
             try:
-                open(md5Name, 'w').write('%s\n' % self.__md5sum)
+                open(md5Name, 'wb').write('%s\n' % self.__md5sum)
             except BaseException as exception:
-                logException(m18n('cannot write <filename>%1</filename>: %2', md5Name, str(exception)))
+                logException(
+                    '\n'.join([m18n('cannot write <filename>%1</filename>: %2',
+                                    md5Name,
+                                    str(exception)),
+                               m18n('The voice files have changed, their checksum has changed.'),
+                               m18n('Please reinstall kajongg or do, with sufficient permissions:'),
+                               'cd {} ; cat *.ogg | md5sum > md5sum'.format(self.directory)]))
         if archiveExists:
-            archiveIsOlder = os.path.getmtime(md5Name) > os.path.getmtime(self.archiveName())
+            archiveIsOlder = os.path.getmtime(
+                md5Name) > os.path.getmtime(self.archiveName())
             if self.__md5sum != existingMd5sum or archiveIsOlder:
                 os.remove(self.archiveName())
 
@@ -344,7 +383,11 @@ class Voice(object):
         if not os.path.exists(self.archiveName()):
             tarFile = tarfile.open(self.archiveName(), mode='w:bz2')
             for oggFile in self.oggFiles():
-                tarFile.add(os.path.join(self.directory, oggFile), arcname=oggFile)
+                tarFile.add(
+                    os.path.join(
+                        self.directory,
+                        oggFile),
+                    arcname=oggFile)
             tarFile.close()
 
     def archiveName(self):
@@ -358,7 +401,7 @@ class Voice(object):
     def savedmd5Sum(self):
         """returns the current value of the md5sum file"""
         if os.path.exists(self.md5FileName()):
-            return open(self.md5FileName(), 'r').readlines()[0].strip()
+            return open(self.md5FileName(), 'r').readlines()[0].replace(' -', '').strip()
 
     @property
     def md5sum(self):
@@ -376,7 +419,7 @@ class Voice(object):
         tarFile = tarfile.open(mode='r|bz2', fileobj=filelike)
         tarFile.extractall(path=self.directory)
         if Debug.sound:
-            logDebug('extracted archive into %s' % self.directory)
+            logDebug(u'extracted archive into %s' % self.directory)
         tarFile.close()
         filelike.close()
 
@@ -385,7 +428,7 @@ class Voice(object):
         """the content of the tarfile"""
         self.__buildArchive()
         if os.path.exists(self.archiveName()):
-            return open(self.archiveName()).read()
+            return open(self.archiveName(), 'rb').read()
 
     @archiveContent.setter
     def archiveContent(self, content):
